/*
 * current_db.c
 *
 * Functions returning info about current database
 *
 * Copyright (c) Joseph Conway <joe.conway@mail.com>, 2001;
 * Licence: BSD
 *
 */

#include "current_db.h"

/*
 * current_dbname
 *
 * Returns name of the current db
 *
 * Borrowed heavily from get_db_info
 * in backend/commands/dbcommands.c
 *
 */

PG_FUNCTION_INFO_V1(current_dbname);
Datum
current_dbname(PG_FUNCTION_ARGS)
{
	char			*dbname;
	Relation		relation;
	ScanKeyData 	scanKey;
	HeapScanDesc	scan;
	HeapTuple		tuple;

	/*
	 * Lookup dbname based on MyProc->databaseId
	 */

	/* Caller may wish to grab a better lock on pg_database beforehand... */
	relation = heap_openr(DatabaseRelationName, AccessShareLock);

	ScanKeyEntryInitialize(&scanKey, 0,ObjectIdAttributeNumber,F_OIDEQ,ObjectIdGetDatum(MyProc->databaseId));

	scan = heap_beginscan(relation, 0, SnapshotNow, 1, &scanKey);
	if (!HeapScanIsValid(scan))
		elog(ERROR, "current_db: Cannot begin scan of %s", DatabaseRelationName);

	tuple = heap_getnext(scan, 0);

	if (HeapTupleIsValid(tuple)) {
		Form_pg_database dbform = (Form_pg_database) GETSTRUCT(tuple);
		dbname = NameStr(dbform->datname);
	} else {
		dbname = NULL;
		elog(ERROR, "current_db: Database not found - oid %d", MyProc->databaseId);
	}

	heap_endscan(scan);
	heap_close(relation, AccessShareLock);

	PG_RETURN_NAME(dbname);
}

/*
 * current_dboid
 *
 * Returns oid of the current db
 *
 */

PG_FUNCTION_INFO_V1(current_dboid);
Datum
current_dboid(PG_FUNCTION_ARGS)
{

	/*
	 * Simply return MyProc->databaseId
	 */

	PG_RETURN_OID(MyProc->databaseId);
}



/*
 * current_dbownername
 *
 * Returns owner name of the current db
 *
 * Borrowed heavily from get_db_info
 * in backend/commands/dbcommands.c
 *
 */

PG_FUNCTION_INFO_V1(current_dbownername);
Datum
current_dbownername(PG_FUNCTION_ARGS)
{
	Oid 			ownerIdP = 0;
	char			*ownername;
	Relation		relation;
	ScanKeyData 	scanKey;
	HeapScanDesc	scan;
	HeapTuple		tuple;

	/*
	 * Lookup dbname based on MyProc->databaseId
	 */

	/* Caller may wish to grab a better lock on pg_database beforehand... */
	relation = heap_openr(DatabaseRelationName, AccessShareLock);

	ScanKeyEntryInitialize(&scanKey, 0,ObjectIdAttributeNumber,F_OIDEQ,ObjectIdGetDatum(MyProc->databaseId));

	scan = heap_beginscan(relation, 0, SnapshotNow, 1, &scanKey);
	if (!HeapScanIsValid(scan))
		elog(ERROR, "current_db: Cannot begin scan of %s", DatabaseRelationName);

	tuple = heap_getnext(scan, 0);

	if (HeapTupleIsValid(tuple)) {
		Form_pg_database dbform = (Form_pg_database) GETSTRUCT(tuple);
		ownerIdP = dbform->datdba;
	} else {
		elog(ERROR, "current_db: Database not found - oid %d", MyProc->databaseId);
	}

	heap_endscan(scan);
	heap_close(relation, AccessShareLock);

	/*
	 * Lookup ownername based on ownerid
	 */

	tuple = SearchSysCache(SHADOWSYSID, ObjectIdGetDatum(ownerIdP), 0, 0, 0);
	if (!HeapTupleIsValid(tuple)) {
		elog(ERROR, "current_db: invalid owner id - datdba %d", (int) ownerIdP);
	}

	ownername = NameStr(((Form_pg_shadow) GETSTRUCT(tuple))->usename);

	ReleaseSysCache(tuple);

	PG_RETURN_NAME(ownername);
}



/*
 * current_dbownerid
 *
 * Returns owner id of the current db
 *
 * Borrowed heavily from get_db_info
 * in backend/commands/dbcommands.c
 *
 */

PG_FUNCTION_INFO_V1(current_dbownerid);
Datum
current_dbownerid(PG_FUNCTION_ARGS)
{
	Oid 			ownerIdP = 0;
	Relation		relation;
	ScanKeyData 	scanKey;
	HeapScanDesc	scan;
	HeapTuple		tuple;

	/*
	 * Lookup dbname based on MyProc->databaseId
	 */

	/* Caller may wish to grab a better lock on pg_database beforehand... */
	relation = heap_openr(DatabaseRelationName, AccessShareLock);

	ScanKeyEntryInitialize(&scanKey, 0,ObjectIdAttributeNumber,F_OIDEQ,ObjectIdGetDatum(MyProc->databaseId));

	scan = heap_beginscan(relation, 0, SnapshotNow, 1, &scanKey);
	if (!HeapScanIsValid(scan))
		elog(ERROR, "current_db: Cannot begin scan of %s", DatabaseRelationName);

	tuple = heap_getnext(scan, 0);

	if (HeapTupleIsValid(tuple)) {
		Form_pg_database dbform = (Form_pg_database) GETSTRUCT(tuple);
		ownerIdP = dbform->datdba;
	} else {
		elog(ERROR, "current_db: Database not found - oid %d", MyProc->databaseId);
	}

	heap_endscan(scan);
	heap_close(relation, AccessShareLock);

	/*
	 * return ownerid
	 */

	PG_RETURN_INT32(ownerIdP);
}

